#!/bin/bash 
#SBATCH -n 1
#SBATCH -t 10:00:00
#SBATCH -J monthly_nbday
#SBATCH -o /cfu/scratch/%u/slurm/monthly_nbday-%j.err
#SBATCH -e /cfu/scratch/%u/slurm/monthly_nbday-%j.err
set -vx 
#to calculate the number of days over/under the climatological monthly 90th/10th percentil and 10th percentil
#chloe prodhomme 27/02/2014



###### To be modified by the USER ######
varlst=(tasmax)
explst=(ifm cmcc ukmo metfr ecmwf)
til=90

y1=1979
y2=2005

monlst=(05) #start month

###### End of the change for the users ######

SLURM_JOBID=""
exptype="ENSEMBLES" # "ENSEMBLES" #ecearth2.3
NLT=7
rerunpctl=0

comp[10]="lt"
comp[90]="gt"



for exp in ${explst[@]}
do
    work=/scratch/$USER/work/monthly_nbdays$SLURM_JOBID/${exp} 
    mkdir -p $work  || echo "Directory already exists"  
    for var in ${varlst[@]} 
    do
        for monstrt in ${monlst[@]}
        do
	    dirdaystat=/scratch/$USER/work/monthly_pctl$SLURM_JOBID/dailymean/${exp}/${var}_day${daystat}
            mkdir -p $dirdaystat  || echo "Directory already exists"  
           #generate the header 
	    case $exptype in
                "ecearth2.3") 
		    nmemb=10
		    suff="_6hourly"
		    data=/cfunas/exp/ecearth/${exp}/daily_grb/${var}_6hourly
		    dataout=/cfunas/exp/ecearth/${exp}/monthly_statistics/
		    mkdir $dataout
		    if [[ $var == "sfcwind" ]]
		    then
                        ncks -O -h -x -v  uas,latitude,longitude,level_$var /cfunas/exp/ecearth/${exp}/monthly_mean/uas_6hourly/uas_${y1}${monstrt}01.nc ${work}/header.nc
		    else
                        ncks -O -h -x -v  $var,latitude,longitude,level_$var ${data/daily_grb/monthly_mean}/${var}_${y1}${monstrt}01.nc ${work}/header.nc
		    fi
		    ;;
                "ENSEMBLES")
		    nmemb=9
		    data=/home/netapp-clima-shared/MODELS/SMR2595/IC3/ENSEMBLES/${exp}/daily/${var}/
		    dataout=/scratch/$USER/data/ENSEMBLES/seasonal/${exp}/monthly_statistics/
		    mkdir -p $dataout  || echo "Directory already exists"  
		    ncks -x -O -h -v  $var,latitude,longitude $data/${var}_${y1}${monstrt}01.nc ${work}/header-aux.nc 
		    suff=""
		    ncks -O -d time,1,$NLT ${work}/header-aux.nc  ${work}/../header.nc
        #if you want to have a proper time and time_bnd value modify : ${work}/header.nc
		    ;;    
	    esac
	    for year in `seq $y1 $y2`
	    do
    #calculate daily stat in the "ecearth2.3" case
                case $exptype in
		    "ecearth2.3")
                        case $var in 
			    "prlr" )
                                varauxlst="lsp cp"
                                var2=prlr
                                freq=24
                                oldname="LSP"
                                daystat=sum
                                ;;
			    "sfcwind" )
                                varauxlst="uas vas"
                                var2=sfcwind
                                freq=6
                                oldname="U10M"
                                daystat=mean  
                                ;;
			    "tas" )
                                if [[ $til == "10" ]]
                                then
				    daystat=min
                                fi
                                if [[ $til == "90" ]]
                                then
				    daystat=max
                                fi
                                varauxlst="tas"
                                var2=tas$daystat
                                oldname="T2M"
                                freq=6
                                ;;
                        esac
                        
                        for varaux in $varauxlst
                        do          
			    data=/cfunas/exp/ecearth/${exp}/daily_grb/${varaux}_6hourly
			    file=${varaux}_${year}${monstrt}01
			    DATE=${year}${monstrt}01
                #number of timestep
			    step=$(( (($(date -u --date=${DATE}+${NLT}month +%s) - $(date -u --date=${DATE} +%s) ))/(60*60*$freq)))
			    cdo setgridtype,regular $data/${file}.grb $work/${file}.grb
			    cdo -r -t ecmwf -f nc copy $work/$file.grb $work/${file}.nc
                            #loop over the members
			    for r in `seq 1 $nmemb`
			    do
                                ncks -O -d time,$(($r-1)),,$nmemb $work/${file}.nc $work/${file}_r${r}.nc 
                                ncks -O -d time,0,$(($step-1)) $work/${file}_r${r}.nc $work/${file}_r${r}.nc2
                                cdo -r settaxis,$year-${monstrt}-01,00:00,6hour $work/${file}_r${r}.nc2 $work/taxis_$file_r${r}.nc2
                                
                                mv $work/taxis_$file_r${r}.nc2 $work/${file}_r${r}.nc

			    done #r
                #concatenate file
			    ncecat -h -O $work/${file}_r*.nc $work/aux_${file}.nc
			    ncpdq -O -h -a time,record $work/aux_${file}.nc $work/aux_${file}.nc
			    mv $work/aux_${file}.nc $work/$file.nc
			    file=${varaux}_${year}${monstrt}01.nc
			    
                        done
                        if [[ $var == "prlr" ]]
                        then
			    cdo -r add $work/lsp_${year}${monstrt}01.nc $work/cp_${year}${monstrt}01.nc $work/prlr_${year}${monstrt}01.nc  
			    file=prlr_${year}${monstrt}01.nc
			    
                        fi
                        if [[ $var == "sfcwind" ]]
                        then
			    cdo -r mul ${work}/uas_${year}${monstrt}01.nc ${work}/uas_${year}${monstrt}01.nc $work/uas2_${year}${monstrt}01.nc  
			    cdo -r mul ${work}/vas_${year}${monstrt}01.nc ${work}/vas_${year}${monstrt}01.nc $work/vas2_${year}${monstrt}01.nc  
			    cdo -r add $work/uas2_${year}${monstrt}01.nc $work/vas2_${year}${monstrt}01.nc $work/uas2-vas2_${year}${monstrt}01.nc  
			    cdo -r sqrt $work/uas2-vas2_${year}${monstrt}01.nc $work/sfcwind_${year}${monstrt}01.nc
			    file=sfcwind_${year}${monstrt}01.nc
			    
                        fi

                                #calculate min, max sum or mean according to the variable
                        cdo -r day${daystat} $work/${file} $work/day${daystat}_${file}
                        mv  $work/day${daystat}_${file} $work/${file}

                        ncrename -O -v $oldname,$var2 $work/$file
                        #copy daily stat for calculate the climatology
                        mkdir -p $dirdaystat
                        cp $work/$file $dirdaystat/$file
                        
                        ;;

		    

        #in ensemble case just the set the time axis of the daily file
		    "ENSEMBLES")
                        var2=$var
			file=${var}_${year}${monstrt}01.nc
                        cdo -r settaxis,$year-${monstrt}-01,12:00,1day $data/$file $work/taxis_$file
                        file=taxis_$file
			if [[ $(ncdump -h $work/$file ) == *realization* ]]
			then
			    ncks -O -h -x -v realization $work/$file $work/norea_$file
			    file=norea_$file
			fi
			
			if [[ $(ncdump -h $work/$file ) == *reftime* ]]
			then
			    ncrename -d reftime,time  $work/$file
			    ncrename -v reftime,time  $work/$file
			fi
                        cp  $work/$file $dirdaystat/${var}_${year}${monstrt}01.nc
                        ;;
                esac
		rm -rf $work/*
		################# calculate 10 and 90 percentile ###################
		file=${var}_${year}${monstrt}01.nc
                cdo -r monmin $dirdaystat/${file} $work/monmin_${file}
                cdo -r monmax $dirdaystat/${file} $work/monmax_${file}
                
                cdo -r monpctl,$til $dirdaystat/${file} $work/monmin_${file} $work/monmax_${file} $work/q${til}_${file}

		################ set header and variables name properly ############
                file=q${til}_${file}
                
                ncrename -O -h -v ${var2},${var2}_q$til $work/${file} $work/${file}1    
                ncrename -O -h -d lev,ensemble $work/${file}1 $work/${file}2
                cp $work/${file}2 $work/${file}6
                if [[ $exptype == "ecearth2.3" ]]
                then
                    ncrename -O -h -d lon,longitude $work/${file}2 $work/${file}3
                    ncrename -O -h -d lat,latitude $work/${file}3 $work/${file}4
                    ncrename -O -h -v lon,longitude $work/${file}4 $work/${file}5
                    ncrename -O -h -v lat,latitude $work/${file}5 $work/${file}6
                fi
                ncrename -h -v time,leadtime $work/${file}6
                
                

                ncks -A $work/../header.nc  $work/${file}6
                ncecat -O -h $work/${file}6 $work/${file}7
                ncpdq -O -h -a time,record $work/${file}7 $work/${file}8
                ncwa -O  -h -a record $work/${file}8 $work/${file}9
                ncks -x -O -h -v lev $work/${file}9 $work/${file}10
                ncatted -h -a long_name,${var2}_q$til,m,c,"monthly ${til} percentile of ${longname[$var2]}" $work/${file}10
                ncatted -h -O -a history,global,d,, $work/${file}10
                
		mkdir -p $dataout/${var2}_q${til}$suff/

                cp $work/${file}10 $dataout/${var2}_q${til}${suff}/${var2}_q${til}_${year}${monstrt}01.nc
                chmod 770  $dataout/${var2}_q${til}$suff/${var2}_q${til}_${year}${monstrt}01.nc


	    done

	    #### calcul the climatologie ######

	    rm -rf $work/*
	    yaux=1000 #start from 1000 to always have 4 digits
	    for year in `seq $y1 $y2`
	    do    
                file=${var2}_${year}${monstrt}01.nc
		#loop over member
		#in order to make believe to cdo that each member represent a different year
                for r in `seq 1 $nmemb`
                do
		    #select one member
		    fileaux=${var2}_${yaux}${monstrt}01.nc
		    cdo sellevel,$r $dirdaystat/$file $work/${fileaux}
		    #remove level dimension
		    ncwa -O -a lev  $work/${fileaux}  $work/nolev_${fileaux}
		    fileaux=nolev_${fileaux}
		    cdo -r settaxis,$yaux-${monstrt}-01,12:00,1day $work/$fileaux $work/taxis_$fileaux
		    yaux=$(( yaux + 1 ))
                done
	    done
	    cdo mergetime $work/taxis_nolev_${var2}_????${monstrt}01.nc $work/cat_taxis_${var2}_1000-${yaux}_${monstrt}01.nc
	    file_out=cat_taxis_${var2}_1000-${yaux}_${monstrt}01.nc
	    rm -rf $work/*${var2}_????${monstrt}01.nc

	    #calculate the climatological percentil
	    cdo -r ymonmin $work/$file_out $work/ymonmin_${file_out}
	    cdo -r ymonmax $work/$file_out $work/ymonmax_${file_out}
	    climtil=ymonq${til}_${file_out}
	    cdo -r ymonpctl,$til $work/$file_out $work/ymonmin_${file_out} $work/ymonmax_${file_out} $work/$climtil 

	    
	    for year in `seq $y1 $y2`
	    do
		#transform monthly file in daily file
		#for experience with the rigth number of day per month
		#need to be put in the loop for february in the ecearth2.3 case
		#this could be optimise in ENSEMBLE case
                for imon in  `seq 0 $((NLT-1))`
                do
		    climtil_mon=${climtil%.nc}_m$imon.nc
		    DATE=${year}${monstrt}01
		    leadmon=$(date -u --date=${DATE}+${imon}month +%m)
		    cdo selmon,$leadmon $work/$climtil $work/$climtil_mon
		    nbday=$(( (($(date -u --date=${DATE}+$((imon+1))month +%s) - $(date -u --date=${DATE}+$((imon))month +%s) ))/(60*60*24)))
		    #in ensemble case each month has 30days 
		    if [[ $exptype == "ENSEMBLES" ]]
		    then 
                        nbday=30
		    fi
		    echo $nbday
		    for iday in `seq 1 $nbday`
		    do
                        cp $work/$climtil_mon $work/toto$iday.nc        
		    done
		    climtil_day=daily_$climtil_mon
		    ncrcat -O $work/toto*.nc $work/daily_$climtil_mon
		    rm -rf $work/toto*.nc
		    
                done
                climtil_comp=daily_$climtil
                ncrcat -O $work/daily_ymonq${til}_${file_out%.nc}_m*.nc  $work/$climtil_comp
		#recreate artifically 10ens member
                for iens in `seq 1 $nmemb`
                do
		    cp $work/$climtil_comp $work/toto$iens.nc        
                done
                
                ncecat -O $work/toto*.nc $work/ens_$climtil_comp
                climtil_comp=ens_$climtil_comp
                rm -rf $work/toto*
                ncpdq -O -h -a time,record $work/${climtil_comp} $work/${climtil_comp}2
                mv $work/${climtil_comp}2 $work/${climtil_comp}

                
                
		#comparison between the daily min/max/sum and the comparison daily climtil just created
                file=$dirdaystat/${var2}_${year}${monstrt}01.nc
                cdo -r ${comp[til]} $file $work/$climtil_comp $work/${comp[til]}q${til}_${var2}_${year}${monstrt}01.nc
                
		#mean to get the percent of number of day 
                cdo -r monmean  $work/${comp[til]}q${til}_${var2}_${year}${monstrt}01.nc $work/nbdaysq${til}_${var2}_${year}${monstrt}01.nc
                
                file=nbdaysq${til}_${var2}_${year}${monstrt}01.nc

                ncrename -O -h -v ${var2},${var2}_nbdaysq${til}_${y1}-${y2} $work/${file} $work/${file}1    
                ncrename -O -h -d lev,ensemble $work/${file}1 $work/${file}2
                cp $work/${file}2 $work/${file}6
                if [[ $exptype == "ecearth2.3" ]]
                then
		    ncrename -O -h -d lon,longitude $work/${file}2 $work/${file}3
		    ncrename -O -h -d lat,latitude $work/${file}3 $work/${file}4
		    ncrename -O -h -v lon,longitude $work/${file}4 $work/${file}5
		    ncrename -O -h -v lat,latitude $work/${file}5 $work/${file}6
                fi
                ncrename -h -v time,leadtime $work/${file}6

                ncks -A $work/../header.nc  $work/${file}6
                ncecat -O -h $work/${file}6 $work/${file}7
                ncpdq -O -h -a time,record $work/${file}7 $work/${file}8
                ncwa -O  -h -a record $work/${file}8 $work/${file}9
                ncks -x -O -h -v lev $work/${file}9 $work/${file}10
                ncatted -h -a long_name,${var2}_nbdaysq${til}_${y1}-${y2},m,c,"number of day over the climatological ${til} percentile" $work/${file}10
                ncatted -h -O -a history,global,d,, $work/${file}10

                mkdir ${dataout}/${var2}_nbdaysq${til}_${y1}-${y2}$suff

                mv $work/${file}10 ${dataout}/${var2}_nbdaysq${til}_${y1}-${y2}$suff/${var2}_nbdaysq${til}_${y1}-${y2}_${year}${monstrt}01.nc
                chmod 770 ${dataout}/${var2}_nbdaysq${til}_${y1}-${y2}$suff/${var2}_nbdaysq${til}_${y1}-${y2}_${year}${monstrt}01.nc
		
	    done #yearlst
	    rm -rf $work/*.nc*
	    rm -rf $dirdaystat/*.nc*
	done   #monlst
    done #varlst
done #explst
